@@ -1,14 +1,15 @@
-#' @template dbispec
 #' @format NULL
-#' @section Getting started:
+#' @importFrom desc desc_get_deps
+#' @section Definition:
 spec_getting_started <- list(
   package_dependencies = function(ctx) {
-    #' A DBI backend is an R package,
-    pkg <- get_pkg(ctx)
+    #' A DBI backend is an R package
+    pkg_path <- get_pkg_path(ctx)
 
-    pkg_imports <- devtools::parse_deps(pkg$imports)$name
+    pkg_deps_df <- desc_get_deps(pkg_path)
+    pkg_imports <- pkg_deps_df[pkg_deps_df[["type"]] == "Imports", ][["package"]]
 
-    #' which should import the \pkg{DBI}
+    #' which imports the \pkg{DBI}
     expect_true("DBI" %in% pkg_imports)
     #' and \pkg{methods}
     expect_true("methods" %in% pkg_imports)
@@ -20,7 +21,7 @@ spec_getting_started <- list(
 
     #' For better or worse, the names of many existing backends start with
     #' \sQuote{R}, e.g., \pkg{RSQLite}, \pkg{RMySQL}, \pkg{RSQLServer}; it is up
-    #' to the package author to adopt this convention or not.
+    #' to the backend author to adopt this convention or not.
     expect_match(pkg_name, "^R")
   },
 
