@@ -1,12 +1,16 @@
-#' @template dbispec
 #' @format NULL
 spec_sql <- c(
   spec_sql_quote_string,
+  spec_sql_quote_literal,
   spec_sql_quote_identifier,
+  spec_sql_unquote_identifier,
   spec_sql_read_table,
+  spec_sql_create_table,
+  spec_sql_append_table,
   spec_sql_write_table,
   spec_sql_list_tables,
   spec_sql_exists_table,
   spec_sql_remove_table,
+  spec_sql_list_objects,
   spec_sql_list_fields
 )
